'use strict';
/**
 * Fairly complex cache to avoid having to pull from the database every time
 * I want to e.g., insert an entry.
 *
 * TODO: add update operations.
 */
import procnet from 'procnet';
import B from 'bluebird';

export default procnet.injectable((cacheEventBus, formStore) => {
	const cache = {};

	const module = {
		get(systemName) {
			return cache[systemName];
		},
		filter(fn) {
			return Object.keys(cache).filter((systemName) => {
				return fn(cache[systemName], systemName);
			});
		},
		remove(systemName) {
			const t = cache[systemName].type;
			return formStore
				.drop(systemName, t)
				.then(() => {
					delete cache[systemName];
					cacheEventBus.publish({
						type: 'form',
						event: 'remove',
						systemName
					});
				});
		},
		addMaster(name, systemName, language, columns) {
			return formStore.createMaster(name, systemName, language, columns)
				.then(() => {
					const form = {
						systemName,
						name,
						columns,
						type: 'master'
					};
					cache[systemName] = form;
					cacheEventBus.publish({
						type: 'form',
						event: 'update',
						systemName
					});
				});
		},
		addProxy(name, systemName, parent, fields) {
			return formStore
				.createProxy(name, systemName, parent, fields)
				.then(() => {
					const form = {
						systemName,
						name,
						fields,
						type: 'proxy'
					};
					cache[systemName] = form;
					cacheEventBus.publish({
						type: 'form',
						event: 'update',
						systemName,
						form
					});
				});
		}
		// TODO: update operations...
	};

	cacheEventBus.on('update:form', (msg) => {
		cache[msg.systemName] = msg.form;
	});

	cacheEventBus.on('remove:form', (msg) => {
		delete cache[msg.systemName];
	});

	return formStore.list().then((forms) => {
		forms.forEach((form) => {
			cache[form.systemName] = form;
		});
	})
	.thenReturn(module);
});
