'use strict';

/**
 * amqp is more complicated than what I need. I just want a way to keep
 * my caches up to date.
 */
import procnet from 'procnet';
import amqplib from 'amqplib';
import conf from '../config';
import EventEmitter from 'events';
import B from 'bluebird';

export default procnet.injectable(() => {

	const emitter = new EventEmitter();

	const onAmqpMsg = (msg) => {
		const str = msg.content.toString();
		const obj = JSON.parse(str);
		emitter.emit(obj.event + ':' + obj.type, obj);
	};
	const createPub = (ch) => {
		return (message) => {
			const buf = new Buffer(JSON.stringify(message));
			ch.publish('cache', '', buf);
		};
	};

	// TODO: listen for error events on the channel.
	const p = amqplib
		.connect(conf.amqp)
		.then((con) => {
			return con.createChannel();
		})
		.then((ch) => {
			ch.assertExchange('cache', 'fanout', { durable: true });
			// create randomly named queue only for this app server.
			return ch
				.assertQueue('', { exclusive: true })
				.then((q) => {
					// bind to the routing key "cache"
					ch.bindQueue(q.queue, 'cache', '');
					ch.consume(q.queue, onAmqpMsg);
					emitter.publish = createPub(ch);
					return emitter;
				});
		});
	// `Cast` to bluebird promise.
	return B.resolve(p);
});
