'use strict';

import procnet from 'procnet';
import nodemailer from 'nodemailer';
import conf from '../config';
import request from 'request';
import B from 'bluebird';
import R from 'ramda';

const overridesRecipient = R.assoc('to', conf.mail.rewriteRecipient);

const sendHttpFormEmail = (data) => {
	return new B((resolve, reject) => {
		request
			.post({
				url: conf.mail.host,
				auth: {
					user: conf.mail.user,
					pass: conf.mail.pass
				},
				from: R.assoc('from', conf.mail.sender, data)
			}, (err, r, body) => {
				if(err) {
					return reject(err);
				}
				resolve();
			});
	});
};

const createSendSmtpEmail = () => {
	const transport = nodemailer.createTransport({
		auth: {
			user: conf.mail.user,
			pass: conf.mail.pass
		},
		port: conf.mail.port,
		host: conf.mail.host
	});
	return (data) => {
		return new B((resolve, reject) => {
			const payload = R.assoc('from', conf.mail.sender, data);
			transport.sendMail(payload, (err,res) => {
				if(err) {
					return reject(err);
				}
				resolve();
			});
		});
	};
};

const mailOptions = R.pick(['from', 'to', 'text', 'html', 'subject']);

export default procnet.injectable((mailTemplate) => {

	const sendMail = conf.mail.type === 'http-form' ?
		sendHttpFormEmail :
		createSendSmtpEmail();
	
	const send = (opts) => {
		const rendered = R.merge(opts, mailTemplate(opts.template, opts.locals));
		return sendMail(mailOptions(rendered));
	};

	if(conf.mail.rewriteRecipient) {
		return {
			send: R.pipe(R.assoc('to', conf.mail.rewriteRecipient), send)
		};
	}

	return {
		send
	};
});
