'use strict';

import procnet from 'procnet';
import error from '../error';
import R from 'ramda';
import B from 'bluebird';

// TODO: should be configurable through the api.
const sessionTimeout = 1000 * 60 * 60 * 24 * 30;
export default procnet.injectable((
		knex,
		roleCache,
		userRoleCache,
		sessionTokenCache) => {
	return (req, res, next) => {
		const token = req.cookies.session;
		if(R.isNil(token) || token.length === 0) {
			return error('no_session');
		}
		return sessionTokenCache
			.get(token)
			.then((user_id) => {
				if(!user_id) {
					return error('session_invalid');
				}
				req.userId = user_id;
				return userRoleCache.get(user_id);
			})
			.then((roles) => {
				if(!roles) {
					return error('session_invalid');
				}
				return B.map(roles, (role) => {
					return roleCache.get(role);
				});
			})
			.then((perms) => {
				req.perms = R.uniq(perms);
				next();
			});
	};
});
