'use strict';

/**
 * Split these accrosss multiple files to avoid merge conflicts. This
 * unfortunately means the modules are loaded dynamically instead of
 * statically.
 */

import procnet from 'procnet';
import fs from 'fs';
import path from 'path';
import R from 'ramda';

const jsFileReg = /[.]js$/;
const files = fs.readdirSync(__dirname);

const loadFiles = R.map((file) => {
	const m = file.match(jsFileReg);
	if(m && file !== 'index.js') {
		return [
			file.substring(0, m.index),
			require('./' + file).default
		];
	}
});

const jsBaseSchema = R.pipe(
	loadFiles,
	R.filter(R.pipe(R.isNil, R.not)),
	R.fromPairs
);

//export default loadFiles(files);

export default procnet.injectable((knex) => {
	const base = jsBaseSchema(files);
	// these schemas may need a knex instance.
	return R.map((fn) => 
		fn.length === 2 ? R.partialRight(fn, [knex]) : fn,
	base);
});
