'use strict';

// The basic idea here is to create a whitelist for errors. By default, any
// error that happens in the system needs to be filtered out, since these
// may provide information that you don't want to see. In these cases, the
// system falls back to a generic 500 error response. If the error is of
// a specific type that is ok though, this means we can send back some details.

const B = require('bluebird');

const error = (msg, cd) => {
	const code = cd || 400;

	if(typeof code !== 'number') {
		throw new TypeError('Code is ' + typeof code);
	}
	const err = new Error(msg);
	err.code = code;
	// flag-based for now...
	err['@@sysError'] = true;
	return B.reject(err);
};

const withCode = (code) => (msg) => error(msg, code);

error.badRequest = withCode(400);
error.unauthorized = withCode(401);
error.notFound = withCode(404);
// This is going to be a bit more common because of versioning.
error.conflict = withCode(409);
error.isSystemError = (err) => !!err['@@sysError'];

export default error;

