'use strict';
/**
 * Load everything...
 */
import R from 'ramda';
import procnet from 'procnet';
import B from 'bluebird';
import conf from './config';
import knex from 'knex';
import loadApp from './app';
import fs from 'fs';
import path from 'path';
import sourceMapSupport from 'source-map-support';

// Add source map support if in dev mode, otherwise its unecesary overhead.
if(process.env.NODE_ENV === 'development') {
	sourceMapSupport.install();
}

const requires = R.map(R.cond([
	[R.is(String), R.pipe(require, R.prop('default'))],
	[R.T, R.identity]
]));

const loadInjectables = R.pipe(requires, procnet.resolve, B.props);

// TODO: these need to be split into a file which is broken down to avoid
// merge conflicts...
const mappings = {
	// Misc.
	mail: './components/mail',
	mailTemplate: './components/mail-template',
	knex: knex(conf.db),
	withPerms: './components/with-perms',
	cacheEventBus: './components/cache-event-bus',

	// Stores
	userStore: './store/user',
	userActivationStore: './store/user-activation',
	sessionTokenStore: './store/session-token',
	formStore: './store/form',

	// Caches
	//formCache: './cache/form',
	roleCache: './cache/role',
	userRoleCache: './cache/user-role',
	sessionTokenCache: './cache/session-token',

	// Validation
	userValidation: './validation/user'//,
	//formValidatin: './validation/form'

};

const apiDir = path.join(__dirname, 'api');

const restApis = fs.readdirSync(apiDir);

const loadApi = (injectables) => {
	return loadApp((app) => {
		restApis.forEach((restFile) => {
			const mod  = require(path.join(apiDir, restFile));
			const deps = R.map(
				(dep) => injectables[dep],
				procnet.dependencies(mod)
			);
			const router = mod.apply(null, deps);
			const api_path = '/' + path.parse(restFile).name;
			app.use(api_path, router);
		});
	});
};

/**
 * Loads the REST application.
 *
 * @returns {Promise<App>}
 */
const load = (overrides) => {
	// TODO: need to add the handler somewhere that works for the tests.
	//process.on('unhandledRejection', (err) => {
	//	process.stderr.write(err.stack);
	//	process.stderr.write('\n');
	//	process.exit(1);
	//});

	return loadInjectables(R.merge(mappings, overrides || {}))
		.then(loadApi);
};

export default load;

