'use strict';

import procnet from 'procnet';
import crypto from '../promise/crypto';
import config from '../config';

const max_age = config.session.maxAge;

export default procnet.injectable((knex) => {
	return {
		create(user_id) {
			return crypto.randomBytes(16).then((buf) => {
				const token = buf.toString('base64');
				return knex('session_token')
					.insert({
						value: token,
						user_id: user_id
					})
					.thenReturn(token);
			});
		},
		/**
		 * Returns the user id corresponding to the session token.
		 */
		find(token) {
			return knex('session_token')
				.where({ value: token })
				.select()
				.then((rows) => {
					if(!rows[0]) {
						return;
					}
					const tkn = rows[0];
					if(Date.now() - Date.parse(tkn.created) > max_age) {
						return;
					}
					return tkn.user_id;
				});
		},
		clear(user_id) {
			return knex('session_token')
				.del()
				.where('user_id', user_id);
		}
	};
});
