'use strict';

import procnet from 'procnet';
import crypto from '../promise/crypto';

export default procnet.injectable((knex) => {
	return {
		create(user_id) {
			return crypto.randomBytes(16).then((buf) => {
				const token = buf.toString('base64');
				return knex('user_activation')
					.insert({ token: token, user_id })
					.thenReturn(token);
			});
		},

		/**
		 * Returns true if the token is valid for the given user, and deletes it
		 * as it is no longer needed afterwards.
		 */
		verify(user_id, token) {
			return knex('user_activation')
				.where({ token, user_id })
				.select()
				.then((rows) => {
					if(rows.length === 0) {
						return false;
					}
					return knex('user_activation')
						.where({ token, user_id })
						.del()
						.thenReturn(true);
				});
		}
	};
});
