'use strict';

import procnet from 'procnet';
import error from '../error';
import R from 'ramda';
import fs from 'fs';
import path from 'path';

//const procnet = require('procnet');
//const error =  require('../error');
//const R = require('ramda');
//const fs = require('fs');

// Tables which are part of the system.
// TODO: need to check for tables which are part of the base system. e.g.,
// information_schema in postgresql.
const baseTables = fs.readdirSync(path.join(__dirname, '../db'))
	.map((file) => file.replace(/[.]js$/, ''));

const validDbName = R.test(/^[A-z_]$/g);
const validColType = R.contains([
	'textbox',
	'integer',
	'datepicker',
	'datetimepicker',
	'yes/no',
	'radiobutton'
]);

module.exports = procnet.injectable((formCache) => {
	return {

		/**
		 * Validates a new master form. Ensures that there are no collisions, etc.
		 */
		createMaster(systemName, columns) {
			if(R.contains(baseTables, systemName)) {
				return error('table_exists');
			}

			if(!validDbName(systemName)) {
				return error('invalid_table_name');
			}
			const validateCols = (check, prop) => 
				R.all(R.pipe(R.prop(prop), check), columns);

			if(validateCols(validDbName, 'systemName', columns)) {
				return error('invalid_column_name');
			}

			if(validateCols(validColType, 'type', columns)) {
				return error('invalid_column_type');
			}

			if(!formCache.get('systemName')) {
				return error('form_not_exist');
			}
			return true;
		},

		/**
		 * Validates the new entry for the proxy form.
		 */
		createProxy(systemName, parent, fields) {
			if(!validDbName(systemName)) {
				return error('invalid_table_name');
			}
			if(formCache.get(systemName)) {
				return error('form_already_exist');
			}
			const parentForm = formCache.get(parent);
			if(!parentForm) {
				return error('form_parent_not_exist');
			}
			if(parentForm.type !== 'master') {
				return error('parent_form_not_exist');
			}

			const parentFields = R.map(R.prop('type'), parentForm.fields);
			const containsField = R.contains(parentFields);
			if(!R.all(containsField, fields)) {
				return error('parent_fields_not_exist');
			}
			return true;
		},

		drop(systemName) {
			const form = formCache.get(systemName);
			if(!form) {
				return error('form_not_exist');
			}
			if(form.type === 'master') {
				return true;
			}
			const proxies = formCache.filter((form) => {
				return form.type === 'proxy' && form.parent === systemName;
			});
			if(proxies.length > 0) {
				return error('form_depend_master');
			}
			return true;
		}

	};
});
