'use strict';

import dynaTable from '../../lib';
import userFixtures from '../fixtures/user';
import supertest from 'supertest';
import deleteRows from '../../lib/db/delete-rows';
import '../globals';

describe('user api', () => {
	const state = {};

	const request = () => {
		return state.agent;
	};

	before(() => {
		return dynaTable().then((app) => {
			state.app = app;
			state.agent = supertest.agent(app.internal);
			return userFixtures();
		});
	});

	it('wont allow me to register with a bad password', () => {
		return request()
			.post('/user/register')
			.send({
				email: 'hello@world.com',
				password: 'helloworld',
				name: 'helloworld'
			})
			.expect(400);
	});

	it('allows me to register', () => {
		return request()
			.post('/user/register')
			.send({
				email: 'hello@world.com',
				password: 'Hello#1World',
				name: 'helloworld'
			})
			.expect(200);
	});

	it('fails login', () => {
		return request()
			.post('/user/login')
			.send({
				email: 'foob@bar.com',
				password: 'WRONG'
			})
			.expect(404);
	});
	it('login', () => {
		return request()
			.post('/user/login')
			.send({
				email: 'foo@bar.com',
				password: 'foobar'
			})
			.expect(200)
			.expect(({body}) => {
				expect(body.csrf).to.exist;
				state.csrf = body.csrf;
			});
	});

	it('should fail if csrf token is missing', () => {
		return supertest(state.app.internal)
			.get('/user/login')
			.expect(400);
	});

	it('has session', () => {
		return request()
			.get('/user/login')
			.set('X-Csrf-Token', state.csrf)
			.expect(200)
			.expect(({body}) => {
				expect(body.name).to.be.equal('foobar');
			});
	});


	after(() => deleteRows());
});
