'use strict';

/**
 * This file is run right before any tests are run. This allows me to override
 * the describe function to allow me to access the `this` context of mocha from
 * arrow functions.
 * 
 * Also adding chai to the TESTING environment's globals as its practically
 * already a global; there is no file in here where I don't require the thing
 * at the top anyways.
 */

import sourceMapSupport from 'source-map-support';
import chai from 'chai';
import chaiAsPromised from 'chai-as-promised';

// jscs:disable


chai.use(chaiAsPromised);
const mochaDescribe = GLOBAL.describe;
const wrapDescribe = (describe) => {
	return (txt, block) => {
		if(!txt) return;
		return describe(txt, function() {
			block(this);
		});
	};
};

const wrapper = wrapDescribe(mochaDescribe);
wrapper.skip = wrapDescribe(mochaDescribe.skip);
wrapper.only = wrapDescribe(mochaDescribe.only);
global.expect = chai.expect;
Object.defineProperty(global, 'describe', {
	get () {
		return wrapper;
	},
	set () {
	}
});

// jscs:enable
