'use strict';

import '../../globals';
import conf from '../../../lib/config';
import knexJs from 'knex';
import userStoreInjectable from '../../../lib/store/user';
import deleteRows from '../../../lib/db/delete-rows';

const knex = knexJs(conf.db);
const userStore = userStoreInjectable(knex);

describe('user store', () => {
	const state = {};

	describe('create', () => {
		before(() => {
			return userStore
				.create('foo', 'foo@bar.com', 'foobar', [], true)
				.then(() => {
					return knex('user')
						.where('name', 'foo')
						.select();
				})
				.then(([usr]) => {
					state.user = usr;
				});
		});

		it('is in the db', () => {
			expect(state.user).to.exist;
		});

		it('has a hashed password', () => {
			expect(state.user.password).to.not.equal('foobar');
		});
		
		it('contains no roles, is array', () => {
			expect(state.user.roles).to.have.lengthOf(0);
		});

	});

	describe('find', () => {
		it('should be able to find by single property', () => {
			return userStore
				.find('id', state.user.id)
				.then((user) => {
					expect(user).to.exist;
				});
		});
		it('should be able to find by compound property', () => {
			return userStore
				.find({
					name: 'foo',
					email: 'foo@bar.com'
				})
				.then((user) => {
					expect(user).to.exist;
				});
		});
	});

	describe('failLogin', () => {
		it('should update the fail date', () => {
			return userStore
				.failLogin(state.user.id)
				.then(() => {
					return knex('user')
						.where('id', state.user.id)
						.select();
				})
				.then(([user]) => {
					expect(user.last_failed_login).to.exist;
				});
		});

		it('should increment the failed logins', () => {
			return userStore
				.failLogin(state.user.id)
				.then(() => {
					return knex('user')
						.where('id', state.user.id)
						.select();
				})
				.then(([user]) => {
					expect(user.failed_logins).to.be.equal(2);
				});
		});
	});

	describe.skip('succeedLogin', () => {
		before(() => {
			return userStore
				.succeedLogin(state.user.id);
		});
		it('should reset the counter to 0', () => {
			return userStore
				.find('id', state.user.id)
				.then((user) => {
					expect(user.failed_logins).to.be.equal(0);
				});
		});
	});

	after(() => deleteRows());
});
